#!/bin/bash
#
# Sample script, to show the way to get plugin running within sample OOTB store.
#
#
# Prerequisites:
#
# Package of Hybris-1808-2011
# Installed Oracle JDK-8, 64-bit / JDK-11, 64-bit for 1905 and higher
# Configured JAVA_HOME
# Configured Ant-1.9.1+
#
# Make sure you don't have running Hybris.
#

function versionFromZip {
unzip -p $HYBRIS_PACKAGE hybris/bin/platform/build.number>build_number.txt

mapfile -t linesFromFile < build_number.txt

SUB='version.api'

for ix in ${!linesFromFile[*]}
do
    if [[ "${linesFromFile[$ix]}" == *"$SUB"* ]]; then
    VERSION=$(echo ${linesFromFile[$ix]} | sed 's/[^0-9]*//g')
    fi
done

echo $VERSION

rm build_number.txt
return $(( VERSION))
}

jdk_version() {
  local result
  local java_cmd
  if [[ -n $(type -p java) ]]
  then
    java_cmd=java
  elif [[ (-n "$JAVA_HOME") && (-x "$JAVA_HOME/bin/java") ]]
  then
    java_cmd="$JAVA_HOME/bin/java"
  fi
  local IFS=$'\n'
  # remove \r for Cygwin
  local lines=$("$java_cmd" -Xms32M -Xmx32M -version 2>&1 | tr '\r' '\n')
  if [[ -z $java_cmd ]]
  then
    result=no_java
  else
    for line in $lines; do
      if [[ (-z $result) && ($line = *"version \""*) ]]
      then
        local ver=$(echo $line | sed -e 's/.*version "\(.*\)"\(.*\)/\1/; 1q')
        # on macOS, sed doesn't support '?'
        if [[ $ver = "1."* ]]
        then
          result=$(echo $ver | sed -e 's/1\.\([0-9]*\)\(.*\)/\1/; 1q')
        else
          result=$(echo $ver | sed -e 's/\([0-9]*\)\(.*\)/\1/; 1q')
        fi
      fi
    done
  fi
  echo "$result"
}

function chooseHybrisVersion {

echo " your previous version was: $HYBRIS_VERSION "
echo "Choose new Hybris version "
options=("1808" "1811" "1905" "2005" "2011")
select HYBRIS_VERSION in "${options[@]}"
do
    case $HYBRIS_VERSION in
        "1808")
            break            
            ;;
        "1811")
            break
            ;;
        "1905")
            break
            ;;
        "2005")
            break
            ;;
	"2011")
            break
            ;;
        *) echo "invalid option $REPLY";;
    esac
done
 echo " you choose $HYBRIS_VERSION "

}

function enterZipAndFolder {
echo "Enter path to Hybris package with name (start with '/' before path), use tab for completion: "
read -e HYBRIS_PACKAGE

echo "Enter output folder name [hybris]: "
read -e HYBRIS_DIR
HYBRIS_DIR=${HYBRIS_DIR:-hybris}

}


function enterZipForVersion {
echo "Enter new HYBRIS.ZIP package path and name for version $HYBRIS_VERSION, use tab for completion: "
read -e HYBRIS_PACKAGE
echo "New package is: $HYBRIS_PACKAGE"
}

function enterOprion {
options=("yes" "no")
select OPTION in "${options[@]}"
do
    case $OPTION in
        "yes")
            break            
            ;;
        "no")
            break
            ;;
        *) echo "invalid option $REPLY";;
    esac
done
}

function chooseAcceleretor {
PS3='Choose Hybris accelerator '
options=("b2c" "b2b")
select HYBRIS_ACCELERATOR in "${options[@]}"
do
    case $HYBRIS_ACCELERATOR in
        "b2c")
            break            
            ;;
        "b2b")
            break
            ;;
        *) echo "invalid option $REPLY";;
    esac
done
}

function installHybris {

echo "starting installation $HYBRIS_VERSION-$HYBRIS_ACCELERATOR"

echo Removing installation...
rm -fr $HYBRIS_DIR

echo Unzipping package...
unzip -q $HYBRIS_PACKAGE -d $HYBRIS_DIR

if [ "$HYBRIS_VERSION" == "1808" ] || [ "$HYBRIS_VERSION" == "1811" ] 
then
if [ "$HYBRIS_ACCELERATOR" == "b2b" ] 
then
cd $HYBRIS_DIR/installer
./install.sh -r b2b_acc
cd -
fi
fi

if [ "$HYBRIS_VERSION" == "1905" ] && [ "$HYBRIS_ACCELERATOR" == "b2b" ]
then
cd $HYBRIS_DIR/installer
./install.sh -r b2b_acc_plus -A local_property:initialpassword.admin=nimda
cd -
fi

if [ "$HYBRIS_VERSION" == "2005" ] || [ "$HYBRIS_VERSION" == "2011" ]
then
cd $HYBRIS_DIR/installer
./install.sh -r cx -A local_property:initialpassword.admin=nimda
cd -
fi

cd $HYBRIS_DIR/hybris/bin/platform
echo develop | ant createConfig
cd -

rm $HYBRIS_DIR/hybris/config/localextensions.xml
rm $HYBRIS_DIR/hybris/config/local.properties
rm -rf $HYBRIS_DIR/hybris/bin/custom-paypal

cp -R ../extensions $HYBRIS_DIR/hybris/bin/custom-paypal
cp -r $HYBRIS_VERSION-$HYBRIS_ACCELERATOR/localextensions.xml $HYBRIS_DIR/hybris/config/localextensions.xml
cp -r local.properties $HYBRIS_DIR/hybris/config/local.properties

cd $HYBRIS_DIR/hybris/bin/platform

if [ "$HYBRIS_ACCELERATOR" == "b2b" ]
then
if [ "$HYBRIS_VERSION" == "1808" ]
then
ant addonuninstall -Daddonnames="smarteditaddon" -DaddonStorefront.yacceleratorstorefront="yb2bacceleratorstorefront"
ant addoninstall -Daddonnames="braintreeb2baddon" -DaddonStorefront.yacceleratorstorefront="yb2bacceleratorstorefront"
fi
if [ "$HYBRIS_VERSION" == "1811" ]
then
ant addoninstall -Daddonnames="braintreeb2baddon" -DaddonStorefront.yacceleratorstorefront="yb2bacceleratorstorefront"
fi
if [ "$HYBRIS_VERSION" == "1905" ]
then
ant addonuninstall -Daddonnames="smarteditaddon" -DaddonStorefront.yacceleratorstorefront="yb2bacceleratorstorefront"
ant addoninstall -Daddonnames="braintreeb2baddon" -DaddonStorefront.yacceleratorstorefront="yb2bacceleratorstorefront"
fi

else 
if [ "$HYBRIS_VERSION" == "1808" ]
then
ant addoninstall -Daddonnames="braintreeaddon" -DaddonStorefront.yacceleratorstorefront="yacceleratorstorefront"
fi
if [ "$HYBRIS_VERSION" == "1811" ]
then
ant addoninstall -Daddonnames="braintreeaddon" -DaddonStorefront.yacceleratorstorefront="yacceleratorstorefront"
fi
if [ "$HYBRIS_VERSION" == "1905" ]
then
ant addonuninstall -Daddonnames="profiletagaddon" -DaddonStorefront.yacceleratorstorefront="yacceleratorstorefront"
ant addoninstall -Daddonnames="braintreeaddon" -DaddonStorefront.yacceleratorstorefront="yacceleratorstorefront"
fi

fi

if [ "$HYBRIS_VERSION" == "2005" ] || [ "$HYBRIS_VERSION" == "2011" ]
then
ant addonuninstall -Daddonnames="profiletagaddon" -DaddonStorefront.yacceleratorstorefront="yacceleratorstorefront"
ant addoninstall -Daddonnames="braintreeaddon" -DaddonStorefront.yacceleratorstorefront="yacceleratorstorefront"
ant addoninstall -Daddonnames="braintreeb2baddon" -DaddonStorefront.yacceleratorstorefront="yb2bacceleratorstorefront"
fi

ant clean all initialize

./hybrisserver.sh
cd -

}

enterZipAndFolder

echo "You selected $HYBRIS_PACKAGE as package "
echo "You selected $HYBRIS_DIR as output folder " 

HYBRIS_VERSION=$( versionFromZip)
java_version="$(jdk_version)"

if [ "$HYBRIS_VERSION" == "2005" ] || [ "$HYBRIS_VERSION" == "2011" ]
then
HYBRIS_ACCELERATOR="cx"
fi

if [ "$HYBRIS_VERSION" -le 1811 ] && [ "$java_version" -gt 8 ];
then
echo "unsupported java version for  hybris v.$HYBRIS_VERSION. Install java 8 for hybris versions less than 1811"
exit -1

elif [ "$HYBRIS_VERSION" -ge 1905 ] && [ "$java_version" -lt 11 ];
then
echo "unsupported java version for  hybris v.$HYBRIS_VERSION. Install java 11 for hybris versions greater than 1905"
exit -1

else
echo "success"
fi

echo "You selected hybris v.$HYBRIS_VERSION. CX recipes will be installed. Continue? yes/no"

enterOprion

if [ "$OPTION" == "yes" ]
then

if [ "$HYBRIS_VERSION" -le 1905 ]
then
chooseAcceleretor
fi

installHybris

else 
chooseHybrisVersion

if [ "$HYBRIS_VERSION" -le 1905 ]
then
chooseAcceleretor
fi

enterZipForVersion
check=$( versionFromZip)

while [ "$check" != $HYBRIS_VERSION ]; do
   echo "entered zip not equals to choosed Hybris version, please enter right version"
   enterZipForVersion
   check=$( versionFromZip)
done
installHybris
fi
